package samples.testObjectGenerator;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.Component;
import xcat.framework.ccacore.PortInfoImpl;
import xcat.framework.ccacore.PortInfo;
import samples.idl.testObject.UsesTestObject;
import xcat.framework.ccacore.UsesPortMapping;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import samples.idl.testObject.TestObject;

/**
 * @version $Revision: 1.1.1.1 $  $Author: mgovinda $ $Date: 2001/05/08 20:23:49 $ (GMT) 
 * @author Rachana Ananathakrishnan [mailto:ranantha@extreme.indiana.edu]
 */

/**
 * The testObjectGenerator component generates int, float, ...,
 * objects and arrays and writes them to its uses port
 */
public class TestObjectGeneratorComponent implements Component  {
  private Services objectGeneratorCore;
  private static Logger logger = Logger.getLogger();
  
  /**
   * An empty constructor for the component is required
   * as reflection is used to instantiated an instance of
   * this class using this constructor, and then the
   * setServices methods is called on it. This is
   * done in SoapComponentInstantiator.java
   */
  public TestObjectGeneratorComponent() {
  }
  
  /**
   * @param services is an instance of ServicesImpl
   * this method is part of the Component interface
   */
  public void setServices(Services cc){
    objectGeneratorCore = cc;
    try {
      // This mapping is used to created the dynamic proxy 
      // to the uses port. This mapping should be added 
      // before the registerUsesPort(...) call is made
      UsesPortMapping.addMapping("testObjectUsesPort",
				 "http://www.extreme.indiana.edu/xcat/samples/wsdl#testObject",
				 new Class[] {UsesTestObject.class}
				 );
      
      // standard cca call to register uses port
      objectGeneratorCore.registerUsesPort(new PortInfoImpl("testObjectUsesPort", "http://www.extreme.indiana.edu/xcat/samples/wsdl#testObject")); 
    } catch(Exception e) {
      e.printStackTrace();
      logger.severe("ObjectGeneratorComponent: unable to register: ", e);
    }
    // start an anonymous thread to wait
    (new Thread() {
	public void run() {
	  doStuff();
	}
      }).start();
  }

  /**
   * wait for 5 seconds before executing
   * this is a substitute till the "go" port
   * is implemented.
   */
  public void doStuff() {
    try {
      synchronized(this) {
	wait(5000);
	startExecuting();
      }
    } catch(InterruptedException ex) {
    }
  }

  /**
   * This method represents whatever computation 
   * the component wants to do
   * Invoke all the methods on the uses port and 
   * check return values.
   */
  public void startExecuting() {
    // the mapping in setServices used UsesTestObject.class
    // and hence UsesString is used here.
    UsesTestObject usesTestObject = null;    
    try {
      logger.finer("ObjectGenerator Component:: execute");
      // "testObjectUsesPort" is the same as the name of the
      // port added in registerUsesPort
      usesTestObject = (UsesTestObject)objectGeneratorCore.getPort("testObjectUsesPort");
      logger.fine("retrieved uses port for testObjectUsesPort " + usesTestObject.getClass());
    } catch (Exception e) {
      e.printStackTrace();
    }

    // now invoke methods on the usesport
    // Since this is a test, a failure in one
    // of the methods should not stop the other 
    // invocations. As a result, each test is 
    // executed in a different try-catch
    try {
      logger.fine("invoking method on the uses port int param 10");
      int intParam = 10;
      Integer retInt = usesTestObject.echoInt(new Integer(intParam));
      logger.fine("done invoking method on the uses port: return value is " + retInt);
      if (retInt.intValue() == intParam)
	System.out.println("echoInt() worked");
      else
	System.out.println("echoInt() failed");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    
    try {
      logger.fine("invoking method on the uses port long param 20");
      long longParam = 10;
      Long retLong = usesTestObject.echoLong(new Long(longParam));
      logger.fine("done invoking method on the uses port: return value is " + retLong );
      if (retLong.longValue() == longParam)
	System.out.println("echoLong() worked");
      else
	System.out.println("echoLong() failed");
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
    logger.fine("invoking method on the uses port short param 30");
      short shortParam = (short)30;
      Short retShort = usesTestObject.echoShort(new Short(shortParam));
      logger.fine("done invoking method on the uses port: return value is " + retShort);
      if (retShort.shortValue() == shortParam)
	System.out.println("echoShort() worked");
      else
	System.out.println("echoShort() failed");
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("invoking method on the uses port double param 40.10");
      double doubleParam = 40.10;
      Double retDouble = usesTestObject.echoDouble(new Double(doubleParam));
      logger.fine("done invoking method on the uses port: return value is " + retDouble);
      if (retDouble.doubleValue() == doubleParam)
	System.out.println("echoDouble() worked");
      else
	System.out.println("echoDouble() failed");
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    try {
      logger.fine("invoking method on the uses port float param 30.10");
      float floatParam = (float)30.10;
      Float retFloat = usesTestObject.echoFloat(new Float(floatParam));
      logger.fine("done invoking method on the uses port: return value is " + retFloat);
      if (retFloat.floatValue() == floatParam)
	System.out.println("echoFloat() worked");
      else
	System.out.println("echoFloat() failed");
    } catch (Exception e) {
      e.printStackTrace();
    }  
    
    try {
      logger.fine("invoking method on the uses port string param Hello there");
      String stringParam = "Hello there";
      String retString = usesTestObject.echoString(stringParam);
      logger.fine("done invoking method on the uses port: return value is " + retString);
      if (stringParam.compareTo(stringParam) == 0)
	System.out.println("echoString() worked");
      else
	System.out.println("echoString() failed");
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("invoking method on the uses port TestObject param 210379, Hello again");
      TestObject objectParam = new TestObject(210379,"Hello again");
      TestObject retObject = usesTestObject.echoTestObject(objectParam);
      logger.finest("done invoking method on the uses port: return value is TestObject" + retObject.toString());
      if (objectParam.equals(retObject))
	System.out.println("echoTestObject() worked");
      else {
	System.out.println("echoTestObject() failed");
	System.out.println(objectParam.toString());
	System.out.println(retObject.toString());
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    try {
      logger.fine("invoking method on the uses port parameter integer array");
      int integerParam[] = new int[5];
      int retParam[] = new int[5];
      for (int i = 0; i < 5; i++)
	  integerParam[i] = i;
      retParam  = usesTestObject.echoIntArray(integerParam);
      int i;
      for (i=0; i<5; i++){
	if (integerParam[i] != retParam[i])
	  break;
      }
      if (i >= 5)
	System.out.println("echoIntArray() worked");
      else
	System.out.println("echoIntArray() failed");	
    } catch (Exception e) {
      e.printStackTrace();
    }

    /*    try{
      logger.fine("invoking method on the uses port parameter short array");
      short shortParam[] = new short[5];
      short retParam[] = new short[5];
      for (int i = 0; i < 5; i++)
	  shortParam[i] = (short)i;
      retParam  = usesTestObject.echoShortArray(shortParam);
      int i;
      for (i=0; i<5;i++){
	if (shortParam[i] != retParam[i])
	  break;
      }
      if (i >= 5)
	System.out.println("Short array worked");
      else
	System.out.println("Short array failed");	
    }catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("invoking method on the uses port parameter long array");
      long longParam[] = new long[5];
      long retParam[] = new long[5];
      for (int i = 0; i < 5; i++)
	  longParam[i] = (long)i;
      retParam  = usesTestObject.echoLongArray(longParam);
      int i;
      for (i=0; i<5;i++){
	if (longParam[i] != retParam[i])
	  break;
      }
      if (i >= 5)
	System.out.println("Long array worked");
      else
	System.out.println("Long array failed");	
    }catch (Exception e) {
      e.printStackTrace();
    }
    */

    try {
      logger.fine("invoking method on the uses port parameter double array");
      double doubleParam[] = new double[5];
      double retParam[] = new double[5];
      for (int i = 0; i < 5; i++)
	  doubleParam[i] = (double)i;
      retParam  = usesTestObject.echoDoubleArray(doubleParam);
      int i;
      for (i=0; i<5;i++){
	if (doubleParam[i] != retParam[i])
	  break;
      }
      if (i >= 5)
	System.out.println("echoDoubleArray() worked");
      else
	System.out.println("echoDoubleArray() failed");	
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("invoking method on the uses port parameter float array");
      float floatParam[] = new float[5];
      float retParam[] = new float[5];
      for (int i = 0; i < 5; i++)
	  floatParam[i] = (float)i;
      retParam  = usesTestObject.echoFloatArray(floatParam);
      int i;
      for (i=0; i<5;i++){
	if (floatParam[i] != retParam[i])
	  break;
      }
      if (i >= 5)
	System.out.println("echoFloatArray() worked");
      else
	System.out.println("echoFloatArray() failed");	
    } catch (Exception e) {
      e.printStackTrace();
    }

    try { 
      logger.fine("invoking method on the uses port parameter String array");
      String StringParam[] = new String[5];
      String retParam[] = new String[5];
      for (int i = 0; i < 5; i++)
	  StringParam[i] = new String("Param"+ i);
      retParam  = usesTestObject.echoStringArray(StringParam);
      int i;
      for (i=0; i<5;i++){
	if (StringParam[i].compareTo(retParam[i]) != 0)
	  break;
      }
      if (i >= 5)
	System.out.println("echoStringArray() worked");
      else
	System.out.println("echoStringArray() failed");	
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("invoking method on the uses port parameter TestObject array");
      TestObject[] TestObjectParam = new TestObject[5];
      TestObject retParam[] = new TestObject[5];
      for (int i = 0; i < 5; i++)
	  TestObjectParam[i] = new TestObject(i,new String("Param"+i));
      retParam  = usesTestObject.echoTestObjectArray(TestObjectParam);
      int i;
      for (i=0; i<5;i++){
	if (!((TestObjectParam[i]).equals(retParam[i])))
	  break;
      }
      if (i >= 5)
	System.out.println("echoTestObjectArray() worked");
      else
	System.out.println("echoTestObjectArray() failed");	
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      logger.fine("done invoking all methods on the uses port");
      objectGeneratorCore.releasePort("testObjectUsesPort");
      logger.fine("Component:: startExecuting done ");
    } catch (Exception e) {
      System.out.println("releasePort(testObjectUsesPort) did not work");
      e.printStackTrace();
    }
  }
}





