/**
 * This class implements the all the methods
 * that will be invoked on ComponentID.
 * Note that some methods are invoked by
 * the user as part of ComponentID interface
 * and some by the framework as part of the
 * ComponentIDAndConnectionAndRegistry interface.
 * ComponentID uses the "Delegation" pattern. It delegates 
 * all the functionality to an implementation of the Service object.
 */

/**
 * @version $Revision: 1.6 $ $Author: mgovinda $ $Date: 2001/05/08 23:43:06 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */
 
package xcat.framework.ccacore;

import soaprmi.server.UnicastRemoteObject;
import soaprmi.RemoteException;
import soaprmi.Remote;
import xcat.framework.exceptions.XCATException;
import soaprmi.util.Logger;
import soaprmi.util.Level;

public class ComponentIDImpl extends UnicastRemoteObject 
  implements ComponentIDAndConnectionAndRegistry {
  
  private ServicesAndServicesConnection servicesImpl;
  private String ref;
  private static Logger logger = Logger.getLogger();

  /** 
   * This constructor is called by ServicesImpl.getComponentID()
   * This ensures that a ComponentId always points to an
   * implementation of the Services interface. The componentID
   * in our implementation is really a proxy for the Services
   * implementation that points to remote processes.
   */
  public ComponentIDImpl(ServicesImpl _servicesImpl) throws RemoteException {
    super();
    logger.finest("called");
    servicesImpl = _servicesImpl;
    //    ref = this.toMyString();
  }
  
  /**
   * This constructor is used for creating a componentID given
   * a stringified reference form for it. The new componentID
   * that is created points to the same services interface
   * implementation as the stringified reference. 
   */

  public ComponentIDImpl(String ref_) throws RemoteException {
    super();
    logger.finest("ref: " + ref_);
    // need to do sanity check on the reference and
    // throw an exception if need be
    ref = ref_;
    ComponentIDAndConnectionAndRegistry componentIDImpl = null;
    try {
      componentIDImpl = 
	ComponentIDFactory.create(ref_);
    } catch (XCATException xcate) {
      logger.severe("Exception in ComponentIDImpl constructor ", xcate);
      throw new RemoteException(xcate.getMessage());
    }
    this.servicesImpl = componentIDImpl.getServicesImpl();
  }

  
  /**
   * needed by the constructor ComponentIDImpl(String ref_)
   * so that it can initialize a services object. 
   */
  public ServicesAndServicesConnection getServicesImpl() 
    throws RemoteException {
    return servicesImpl;
  }

  /**
   * This method is part of the ComponentIDprivate interface.
   * This method is called by ConnectionServiceImpl::connect(...)
   * The Connection service calls this method when it wants to
   * connect a remote provides port to the userPort of the this
   * component. ProvidesPort reference that has been passed in
   * here is a remote reference. '
   */
  public void xcatConnect(String usesPortName, ProvidesPort providesPort) 
    throws RemoteException {
    logger.finest("usesPortName: " + usesPortName);
    try {
      servicesImpl.xcatConnect(usesPortName, providesPort);
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatConnect. ";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
  }

  /**
   * This method is part of the ComponentIDprivate interface.
   * This method is called by ConnectionServiceImpl::disconnect(...)
   * The Connection service calls this method when it wants to
   * disconnect a remote provides port to the userPort of the this
   * component. ProvidesPort reference that has been passed in
   * here is a remote reference. 
   */
  public void xcatDisconnect(String usesPortName, ProvidesPort providesPort) 
    throws RemoteException {
    logger.finest("usesPortName: " + usesPortName);
    try {
      servicesImpl.xcatDisconnect(usesPortName, providesPort);
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatDisconnect. ";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
  }

  /**
   * This method is part of the ComponentIDprivate interface.
   * This method is called by ConnectionServiceImpl::connect(...)
   * and ConnectionServiceImpl::disconnect(...). The Connection
   * service obtains a remote reference to a provides port via
   * this call and passes it on to another ComponentID to connect
   * to its uses port.
   */
  public ProvidesPort xcatGetProvidesPort(String providesPortName) 
    throws RemoteException {
    logger.finest("providesPortName: " + providesPortName);
    ProvidesPort providesPort = null;
    try {
      providesPort  =(ProvidesPort) servicesImpl.xcatGetProvidesPort(providesPortName);
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatGetProvidesPort. ";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
    return providesPort;
  }

  /**
   * export the given provides port as my newPortName. 
   */
  public void xcatExportAs(PortInfo newPortInfo, ProvidesPort providesPort) 
    throws RemoteException {
    logger.finest("portInfo: " + newPortInfo.toString());
    try {
      servicesImpl.xcatExportAs(newPortInfo, providesPort);
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatExportAs. ";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
  }
  
  public void xcatProvideTo(String usesPortName, ProvidesPort providesPort) 
    throws RemoteException {
    logger.finest("usesPortName: " + usesPortName);
    try {
      servicesImpl.xcatProvideTo(usesPortName, providesPort);
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatProvideTo ";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
  }
  
  /**
   * get the port type string corresponding to a port name string.
   */
  public String xcatGetPortTypeForPortName(String portName) 
    throws RemoteException {
    logger.finest("portName: " + portName);
    String portType = null;
    try {
      portType = servicesImpl.xcatGetPortTypeForPortName (portName);     
    } catch (XCATException xcate) {
      String exceptionMessage = "Exception caught in ComponentIDImpl::xcatGetPortTypeForPortName";
      logger.severe(exceptionMessage, xcate);
      throw new RemoteException(exceptionMessage + xcate.getMessage());
    }
    return portType;
  }

  /**
   * These methods were added as we
   * need to be able to store and
   * retrieve ComponentIDs from the
   * instance registry. So, the are part
   * of the ComponentIDRegistry interface
   */
  public boolean equals(ComponentIDRegistry cid) throws RemoteException {
    throw new RemoteException("the equals method is not supported as of now");
    // FIXME FIXME
  }
  
  public String toMyString() throws RemoteException {
    logger.log(Level.WARNING, "this method is not yet implemented ");
    throw new RemoteException("the toMyString() method is not supported as of now");
    // FIXME FIXME
  }

  public String getRef() throws RemoteException {
    logger.finest("called");
    return ref;
  }
}











