package xcat.framework.ccacore;

import soaprmi.RemoteException;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import soaprmi.server.UnicastRemoteObject;
import soaprmi.RemoteException;
import java.lang.reflect.Method;

/**
 * @version $Revision: 1.2 $ $Author: mgovinda $ $Date: 2001/05/08 20:06:00 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * This class implements the ProvidesPort for all types.
 * For each type of usesPort registered using 
 * addProvidesPort(...), an instance of this proxy
 * class is instantiated. Before invoking addProvidesPort()
 * the component user is required to add a mapping
 * between the port type and its interface class
 * example, string and ProvidesString.class.
 */
public class ProvidesDynamicSkelImpl extends UnicastRemoteObject
  implements InvocationHandlerAndProvidesPort{
  private static Logger logger = Logger.getLogger();
  protected boolean providesPortImplIsSet = false;
  protected ProvidesPort implOfProvidesPort;

  // a list of known methods that can be invoked
  // on the dynamic proxy. These methods are all
  // implemented in a super class (Object) or
  private static Method hashCodeMethod;
  private static Method equalsMethod;
  private static Method toStringMethod;
  // this method can be invoked as it is part
  // of the ProvidesPortImplInterface interface
  // that this dynamic proxy implements
  private static Method assignProvidesPortImplMethod;


  static {
    // initialize the list of known methods. The static initializer
    // is called just once for this class.
    try {
      assignProvidesPortImplMethod = 
	xcat.framework.ccacore.ProvidesPortImplInterface.class.getMethod("assignProvidesPortImpl", new Class[] {ProvidesPort.class});
      hashCodeMethod = Object.class.getMethod("hashCode", null);
      equalsMethod =
        Object.class.getMethod("equals", new Class[] { Object.class });
      toStringMethod = Object.class.getMethod("toString", null);
    } catch (NoSuchMethodException e) {
      throw new NoSuchMethodError(e.getMessage());
    }
  }

  /**
   * Since this is a remote class, it is required that
   * the superclass constructor be called.
   */
  public ProvidesDynamicSkelImpl() throws RemoteException {
    super();
    logger.finest("called");
  }

  /**
   * This method is called by addProvidesPort() in ServicesImpl.java.
   * The actual implementation instance of the provides port 
   * given in the addProvidesPort(...) call is stored here. 
   */
  public void assignProvidesPortImpl(ProvidesPort thePort) {
    logger.finest("called with port " + thePort.getClass().getName());
    implOfProvidesPort = thePort;
    providesPortImplIsSet = true;
  }
  
  /**
   * @param proxy: the instance on which the method was invoked
   * for our purposes the proxy is not used at all.
   * @param m: the actual method of the interface that was invoked
   * @param args: the list of arguments that were used in the invocation
   * Whenever a method is invoked on any of the interfaces 
   * of this dynamic proxy, the invoke() method is called.
   */
  public Object invoke(Object proxy, Method m, Object[] args) 
    throws Throwable {
    String methodName = m.getName();
    Class className = m.getDeclaringClass();
    
    logger.finest("called with methodName " + methodName);
    // toString, hashCode and equals is called 
    // by soaprmi on a remote ref.
    // Since the remote ref to a provides port
    // is implemented as a dynamic proxy, these methods
    // called on invoke. By invoking it on "this", the
    // method will be invoked on the base class "Object".

    if (m.equals(assignProvidesPortImplMethod)) {
      logger.finest("invoking assignProvidesPortImpl ");
      // To be able to invoke assignProvidesPortImpl(...)
      // as m.invoke(this, args), this class has to implement
      // the interface ProvidesPortImplInterface. Or else
      // it is not possible to invoke the method using m.invoke().
      // In the case that the m.invoke(...) method is not 
      // part of an interface, it should be explicitly called
      // by casing args[0] to ProvidesPort and then invoking 
      // assignProvidesPortImpl(...) on "this" without m.invoke()

      // use this commented out code if assignProvidesPortImpl
      // is not part of the interface that ProvidesDynamicProxySkel
      // implements.
      //        ProvidesPort providesPort = (ProvidesPort)args[0];
      //        assignProvidesPortImpl(providesPort);
      return m.invoke(this, args);
    } else if (m.equals(hashCodeMethod)) {
      return m.invoke(this, args);
    } else if (m.equals(toStringMethod)) {
      return m.invoke(this, args);
    } else if (m.equals(equalsMethod)) {
      return m.invoke(this, args);
    } else {
      if(providesPortImplIsSet){
	return m.invoke(implOfProvidesPort, args);
      } else {
	logger.severe("An implementation for ProvidesPort has not been provided!, check if addProvidesPort(...) has been called");
      }
    }
    return null;
  } // end public Object invoke(Object proxy, Method m, Object[] args) 
}





