package xcat.framework.ccacore;

import xcat.framework.exceptions.XCATException;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
/**
 * @version $Revision: 1.3 $ $Author: mgovinda $ $Date: 2001/05/08 23:43:06 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * A mapping is required between the string names of the provides port
 * types and actual java interface classes. This is needed by 
 * the dynamic proxy used to implement the uses port (ProvidesDynamicStubImpl).
 * This class is used by the component user to add a mapping before
 * invoking addProvidesPort(...)
 */
public class ProvidesPortMapping {
  
  public class ProvidesPortInfoInternal {
    private String providesPortName;
    private String providesPortType;
    private Class[] arrayOfInterfaces;
    
  /**
   * ProvidesPortInfoInternal is an inner class
   * that is used to store all the mapping info
   * related to a particular port name. The
   * inner class is stored in a hash table along
   * with the name of the port.
   */

    public ProvidesPortInfoInternal(String providesPortName_,
			String providesPortType_,
			Class[] arrayOfInterfaces_) {
      providesPortName = providesPortName_;
      providesPortType = providesPortType_;
      arrayOfInterfaces = arrayOfInterfaces_;
    }
			
    public String getProvidesPortName() { return providesPortName; }
    public String getProvidesPortType() { return providesPortType; }
    public Class[] getArrayOfInterfaces() { return arrayOfInterfaces; }
  }
  
  private static Map providesPortMap;
  private static Logger logger = Logger.getLogger();

  static {
    providesPortMap = new HashMap();
  }

  // this static method is called by a component developer
  // before calling addProvidesPort  
  public static void addMapping(String providesPortName,
				String providesPortType,
				Class[] arrayOfInterfaces
				) {
    logger.finest("addMapping for " + providesPortName + " and " + providesPortType);
    ProvidesPortMapping providesPortMapping = new ProvidesPortMapping();
    providesPortMapping.init(providesPortName,
			 providesPortType,
			 arrayOfInterfaces);
  }

  private void init(String providesPortName,
		    String providesPortType,
		    Class[] arrayOfInterfaces
		    ) {
    
    int lengthOfArray = arrayOfInterfaces.length;
    int lengthOfFinalArray = lengthOfArray + 2;
    Class[] newArrayOfInterfaces = new Class[lengthOfFinalArray];
    for (int i=0; i < lengthOfArray; i++) {
      newArrayOfInterfaces[i] = arrayOfInterfaces[i];
    }
    // for the dynamic proxy to work, it needs to have two 
    // interfaces other than the ones that the user adds for the
    // port types.
    // ProvidesPortImplInterface is needed so that the
    // method assingProvidesPortImpl(...) can be invoked on the
    // proxy. soaprmi.server.RemoteRef.class is needed so that
    // when a provides port is returned on a remote call it gets
    // packed as a soaprmi port. 
    newArrayOfInterfaces[lengthOfFinalArray-2] = ProvidesPortImplInterface.class;
    newArrayOfInterfaces[lengthOfFinalArray-1] = soaprmi.server.RemoteRef.class;
    
    ProvidesPortInfoInternal providesPortInfoInternal
      = new ProvidesPortInfoInternal(providesPortName,
				     providesPortType,
				     newArrayOfInterfaces);
    // key = providesPortName
    // value = providesPortInfo
    providesPortMap.put(providesPortName, providesPortInfoInternal);
  }
  
  private static ProvidesPortInfoInternal getProvidesPortInfoInternal(String providesPortName) throws XCATException {
    Iterator iter = providesPortMap.entrySet().iterator();
    while (iter.hasNext()) {
      Map.Entry entry = (Map.Entry)iter.next();
      String portName = (String)entry.getKey();
      if (portName.equals(providesPortName)) {
	return (ProvidesPortInfoInternal)entry.getValue();
      }
    }
    String exceptionMessage = "No mapping is available for providesPort with the following name: " + providesPortName;
    logger.severe(exceptionMessage);
    throw new XCATException(exceptionMessage);
  }
  
  public static String getProvidesPortType(String providesPortName) throws XCATException {
    ProvidesPortInfoInternal providesPortInfoInternal = 
      getProvidesPortInfoInternal(providesPortName);
    String providesPortType = providesPortInfoInternal.getProvidesPortType();
    logger.finest("getProvidesPortType returning " + providesPortType + " for " + providesPortName);
    return providesPortType;
  }
  
  /**
   * At present, providesPortType is not provides to search the map,
   * but is should be. FIXME
   */
  public static Class[] getArrayOfInterfaces(String providesPortName, String providesPortType) throws XCATException {
    ProvidesPortInfoInternal providesPortInfoInternal = 
      getProvidesPortInfoInternal(providesPortName);
    Class[] arrayOfInterfaces = providesPortInfoInternal.getArrayOfInterfaces();
    return arrayOfInterfaces;
  }

} 

