package xcat.framework.ccacore;

import xcat.framework.ccacore.UsesPortImpl;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

/**
 * @version $Revision: 1.2 $ $Author: mgovinda $ $Date: 2001/05/08 20:06:00 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * This class implements the UsesPort for all types.
 * For each type of usesPort registered using 
 * registerUsesPort(...), an instance of this proxy
 * class is instantiated. Before invoking registerUsesPort()
 * the component user is required to add a mapping
 * between the port type and its interface class
 * example, string and UsesString.class.
 */
public class UsesDynamicStubImpl extends UsesPortImpl 
  implements InvocationHandler {
  // a list of known methods that can be invoked
  // on the dynamic proxy. These methods are all
  // implemented in the super class (UsesPortImpl).
  private static Method addListenerMethod;
  private static Method removeListenerMethod;
  private static Method hasAllVoidMethodsMethod;
  private static Method isConnectedMethod;
  private static Method incrRefCountMethod;
  private static Method decrRefCountMethod;
  private static Method getRefCountMethod;
  private static Logger logger = Logger.getLogger();

  /**
   * The code in the static intializer
   * is called exactly once for this class. 
   * Add all the methods on UsesPort to the list of known methods
   * So, in invoke() the method invoked can be compared to this
   * list of known methods and then invoked. If the method is not
   * known then it has to be a remote method and should be 
   * invoked on the remote reference to the provides port that
   * the uses port is connected to. 
   */
  static {
    try {
      addListenerMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("addListener", new Class[] {ProvidesPort.class});
      
      removeListenerMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("removeListener", new Class[] {ProvidesPort.class});
      
      hasAllVoidMethodsMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("hasAllVoidMethods", null);
      isConnectedMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("isConnected", null);
      
      incrRefCountMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("incrRefCount", null);
      
      decrRefCountMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("decrRefCount", null);
      
      getRefCountMethod = 
	xcat.framework.ccacore.UsesPort.class.getMethod("getRefCount", null);
    } catch (NoSuchMethodException e) {
      throw new NoSuchMethodError(e.getMessage());
    }
  }

  /**
   * all the methods on this port do not have void as return values.
   * So, invoke the superclass constructor with "false".
   */
  public UsesDynamicStubImpl() {
    super(false);
    logger.finest("called: ");
  }
  
  /**
   * @param proxy: the instance on which the method was invoked
   * for our purposes the proxy is not used at all.
   * @param m: the actual method of the interface that was invoked
   * @param args: the list of arguments that were used in the invocation
   * Whenever a method is invoked on any of the interfaces 
   * of this dynamic proxy, the invoke() method is called.
   */
  public Object invoke(Object proxy, Method m, Object[] args) 
    throws Throwable {
    String methodName = m.getName();
    Class className = m.getDeclaringClass();
    
    logger.finest("called for methodName " + methodName);
    // if one of the superclass methods (on UsesPort interface)
    // was invoked then invoke the corresponding the superclass
    // method.
    if ( (m.equals(addListenerMethod)) ||
	 (m.equals(removeListenerMethod)) ||
	 (m.equals(hasAllVoidMethodsMethod)) ||
	 (m.equals(incrRefCountMethod)) ||
	 (m.equals(decrRefCountMethod)) ||
	 (m.equals(isConnectedMethod)) ||
	 (m.equals(getRefCountMethod))
	 ) {
      return m.invoke(this, args);
    } else {
      // else the method was invoked on a user specified port type
      // and it needs to invoked on all the listeners of this port
      for(int i = 0; i < (portList.size()-1); i++) {
	ProvidesPort remoteRef = (ProvidesPort) portList.elementAt(i);
	logger.finer("retrieved providesPort" + remoteRef.getClass());
      }
      // just returning the values from the last invocation
      // need to check the CCA spec on which return value to
      // pick up from the list of listeners.
      int j = portList.size() -1;
      ProvidesPort remoteRef = (ProvidesPort) portList.elementAt(j);
      return m.invoke(remoteRef, args);
    }
  }
}
