package xcat.framework.ccacore;

import xcat.framework.exceptions.XCATException;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
/**
 * @version $Revision: 1.3 $ $Author: mgovinda $ $Date: 2001/05/08 23:43:07 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * A mapping is required between the string names of the uses port
 * types and actual java interface classes. This is needed by 
 * the dynamic proxy used to implement the uses port (UsesDynamicStubImpl).
 * This class is used by the component user to add a mapping before
 * invoking registerUsesPort(...)
 */
public class UsesPortMapping {
  
  /**
   * UsesPortInfoInternal is an inner class
   * that is used to store all the mapping info
   * related to a particular port name. The
   * inner class is stored in a hash table along
   * with the name of the port.
   */
  public class UsesPortInfoInternal {
    private String usesPortName;
    private String usesPortType;
    private Class[] arrayOfInterfaces;
    
    public UsesPortInfoInternal(String usesPortName_,
			String usesPortType_,
			Class[] arrayOfInterfaces_) {
      usesPortName = usesPortName_;
      usesPortType = usesPortType_;
      arrayOfInterfaces = arrayOfInterfaces_;
    }
			
    public String getUsesPortName() { return usesPortName; }
    public String getUsesPortType() { return usesPortType; }
    public Class[] getArrayOfInterfaces() { return arrayOfInterfaces; }
  }
  
  // this map is used to store mapping between a 
  // port name, its interfaces and port type.
  private static Map usesPortMap;
  private static Logger logger = Logger.getLogger();

  // using static initializer to initialize
  // a static variable
  static {
    usesPortMap = new HashMap();
  }
  
  // this static method is called by a component developer
  // before calling registerUsesPort
  public static void addMapping(String usesPortName,
				String usesPortType,
				Class[] arrayOfInterfaces
				) {
    logger.finest("addMapping for " + usesPortName + " and " + usesPortType);
    UsesPortMapping usesPortMapping = new UsesPortMapping();
    usesPortMapping.init(usesPortName,
			 usesPortType,
			 arrayOfInterfaces);
  }

  // It is not possible to initialize UsesPortInfoInternal
  // inside a static method. So the method init is used
  // which is not static. 
  private void init(String usesPortName,
		    String usesPortType,
		    Class[] arrayOfInterfaces
		    ) {
    
    UsesPortInfoInternal usesPortInfoInternal
      = new UsesPortInfoInternal(usesPortName,
				 usesPortType,
				 arrayOfInterfaces);
    // key = usesPortName
    // value = usesPortInfo
    usesPortMap.put(usesPortName, usesPortInfoInternal);
  }
  
  private static UsesPortInfoInternal getUsesPortInfoInternal(String usesPortName) throws XCATException {
    Iterator iter = usesPortMap.entrySet().iterator();
    while (iter.hasNext()) {
      Map.Entry entry = (Map.Entry)iter.next();
      String portName = (String)entry.getKey();
      if (portName.equals(usesPortName)) {
	return (UsesPortInfoInternal)entry.getValue();
      }
    }
    String exceptionMessage = "No mapping is available for usesPort with the following name: " + usesPortName;
    logger.severe(exceptionMessage);
    throw new XCATException(exceptionMessage);
  }
  
  public static String getUsesPortType(String usesPortName) throws XCATException {
    UsesPortInfoInternal usesPortInfoInternal = 
      getUsesPortInfoInternal(usesPortName);
    String usesPortType = usesPortInfoInternal.getUsesPortType();
    logger.finest("getUsesPortType returning " + usesPortType + " for " + usesPortName);
    return usesPortType;
  }
  
  /**
   * At present, usesPortType is not uses to search the map,
   * but is should be. FIXME
   */
  public static Class[] getArrayOfInterfaces(String usesPortName, String usesPortType) throws XCATException {
    UsesPortInfoInternal usesPortInfoInternal = 
      getUsesPortInfoInternal(usesPortName);
    Class[] arrayOfInterfaces = usesPortInfoInternal.getArrayOfInterfaces();
    return arrayOfInterfaces;
  }

} 

