package xcat.framework.creation;

import xcat.framework.exceptions.XCATException;
import xcat.framework.ccacore.UsesPort;
import xcat.framework.ccacore.ComponentID;
import xcat.framework.util.EnvObj;
import xcat.framework.util.Fork;
import xcat.framework.util.SoapConnectionServer;
import xcat.framework.util.SoapConnectionServerImpl;
import xcat.framework.util.SoapConnectionClient;
import soaprmi.RemoteException;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.net.InetAddress;
import java.net.UnknownHostException;


/**
 * The creation service receives the creation environment 
 * as a name-value pair in the envObject. Typically the
 * values will be recevied from an XML sepcification for
 * the execution environment for the component.
 * @version $Revision: 1.6 $  $Author: mgovinda $ $Date: 2001/05/08 23:43:08 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */
 
public class CreationServiceImpl implements CreationService {

  private static Logger logger = Logger.getLogger();

  public ComponentID createInstance(EnvObj env) throws XCATException {

    logger.finest("Creation Env is :: " + env.toString());
    
    String localIPAddress = null;
    String creationProto = env.get("creation-proto");
    String creationMechPath = env.get("creation-mech-path");
    String execHost = env.get("exec-host");
    String execPath = env.get("exec-path");
    String execName = env.get("exec-name");
    String execFQN  = env.get("exec-fqn");

    // if for some reason the IP address cannot be 
    // found then use "localhost" as the default. 
    // maybe instead of localhost we should use "127.0.0.1"
    try {
      localIPAddress = InetAddress.getLocalHost().getHostAddress();
    } catch (UnknownHostException e) {
      e.printStackTrace();
      logger.severe("unable to detrmine local host. using 'localhost' instead", e);
      localIPAddress = "localhost";
    }

    // SoapConnectionServer waits on a port to receive the
    // componentID from the instantiated component
    SoapConnectionServerImpl soapConnectionServerImpl = null; 
    try {
      soapConnectionServerImpl = 
	new SoapConnectionServerImpl();
    } catch (RemoteException re) {
      throw new XCATException("could not create component", re);
    }

    // SoapConnectionServer waits on a new port each time. This
    // port information has to be sent to the instantiated component
    // as command line arguments. 
    String portForComponentCallBack = 
      "" + soapConnectionServerImpl.getPortForComponentCallBack();
    logger.finer("portForComponentCallBack is " + portForComponentCallBack);

    // depending on the creation mechanism specified in the envObject
    // call the appropriate creation mechanism method.
    if (creationProto.equals("ssh")) {
      createInstanceUsingSSHorRSH(creationMechPath, execHost, execPath, localIPAddress, portForComponentCallBack, execName, execFQN);
    } else if (creationProto.equals("rsh")) {
      createInstanceUsingSSHorRSH(creationMechPath, execHost, execPath, localIPAddress, portForComponentCallBack, execName, execFQN);
    } else if (creationProto.equals("gram-java")) {
      execName = null;
      createInstanceUsingCog(execHost, execPath, execName, localIPAddress, portForComponentCallBack);
    }  else {
      String exceptionMessage =  "CreationServiceImpl::createInstance instantiation doesn't exist for this protocol: " + creationProto;
      logger.severe("throwing XCATException " + exceptionMessage);
      throw new XCATException(exceptionMessage);
    } 
    
    // wait till the componentID is received from the instantiated component
    ComponentID componentIDofInstantiatedComponent = 
      soapConnectionServerImpl.getComponentID();
    
    logger.finer("received componentID of instantiated component");
    soapConnectionServerImpl.doShutDown(true);
    
    return componentIDofInstantiatedComponent;
  }
 

  /**
   * creationMechPath : example /usr/bin/ssh
   * execGlobusRunHost: run globusrun on this host
   * globusRunScriptPath: path for globusRunScript on the execGlobusRunHost
   * execHost: host on which to exec the c++ component
   * excPath: path to executable directory on the the execHost
   * execName: name of the executable on the exec directory
   * String localIPAddress: host on which this creation service is waiting
   *                   for a callback from the instantiated c++ component
   * String portForComponentCallback: port for same reason as above.
   */
 
  private void createInstanceUsingCog(String execHost, 
				      String execPath, 
				      String execName, 
				      String localIPAddress, 
				      String portForComponentCallBack) 
				      throws XCATException {

//     final String blank = " ";
//     String theContact;
//     String theRSL;
//     GramJob theJob;
   
//     // Allow the user to set their own output dir aside from $(HOME)
//     if(outputDir==null) {
//       String file = System.getProperty("COG_PROPERTIES_FILE");
//       // Allow the user to set their own output dir aside from $(HOME)
//       if(outputDir==null) {
//         String file = System.getProperty("COG_PROPERTIES_FILE");
//         if (file == null) {
//  	 String globus_dir = System.getProperty("user.home") +
//  	   File.separator +".globus" + File.separator;
//  	 file = globus_dir + "cog.properties";
//         }
//         props = org.globus.security.Config.readProperties(file);
//         outputDir=props.getProperty("GRAM_OUTPUT_DIR");
//         if(outputDir == null) outputDir="$(HOME)";
//       }
     
//       theContact=fixHostname(execHost);
     
//       // This sets the RSl string, which we don't really have to do
//       // seperately, just for the sake of printing out for diagnostic
//       // purposes.
     
//       theRSL="&(executable=" + execName + ")(directory=" + execPath + ")(environme
//  nt=(CALLBACK_HOST " + localIPAddress + ")(CALLBACK_PORT " + portForComponentCallBack
//         + ")(PROCESS_SEQ 1))(stdout=" + outputDir + "/ # \"" +execName + ".out\")(stderr
//  =" + outputDir + "/ # \"" +execName + ".err\")(count=1)";
     
//       // Set up the job
//      theJob=new GramJob(theRSL);

//      System.out.println("\nCreationServiceImpl:: rsl is: " + theRSL);
//      System.out.println("\nCreationServiceImpl:: contact is: " + theContact);

//      try {
//        Gram.request(theContact,theJob);
//      } catch (Exception e) {
//        throw new XCATException("Error submitting gram job: " + e.getMessage());
//      }
  }

   
  private void createInstanceUsingSSHorRSH(String creationMechPath, 
					   String execHost, 
					   String execPath, 
					   String creationServiceHost, 
					   String creationServicePort,
					   String execName,
					   String execFQN) {

    final String blank = " ";
    final String fullcmd = creationMechPath + blank + execHost + blank + execPath + blank + creationServiceHost + blank + creationServicePort + blank + execName + blank + execFQN;
    logger.finer(fullcmd);
    (new Fork()).forkProcess(fullcmd);
  }
   
  public void deleteInstance(ComponentID id) throws XCATException {
    logger.finer("called");
    String exceptionMessage = "deleteInstance is not supported as of now";
    logger.severe(exceptionMessage);
    throw new XCATException(exceptionMessage);
    // FIXME FIXME
    // to be implemented
  }
  
}
