package xcat.framework.util;

import xcat.framework.ccacore.ComponentID;
import soaprmi.server.UnicastRemoteObject;
import soaprmi.soaprpc.SoapServices;
import soaprmi.RemoteException;
import soaprmi.util.Logger;
import soaprmi.util.Level;


/**
 * @version $Revision: 1.4 $  $Author: mgovinda $ $Date: 2001/05/08 20:07:06 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

public class SoapConnectionServerImpl extends UnicastRemoteObject 
  implements SoapConnectionServer {

  public final static String SERVICE_NAME = "urn:xcat:creationService";
  // let the system choose a free port
  private final static int CALL_BACK_PORT = 0;
  private boolean gotShutdown = false;
  private boolean gotComponentID = false;
  private ComponentID componentIDofInstantiatedComponent = null;
  private int portWaitingOn;
  private static Logger logger = Logger.getLogger();

  public SoapConnectionServerImpl() throws RemoteException {
    this(CALL_BACK_PORT, SERVICE_NAME);
  }
 
  /**
   * This constructor calls the superclass constructor
   * to make it a remote object that acts as a service
   */
  public SoapConnectionServerImpl(int portNumber, String serviceName) 
    throws RemoteException {
    super(
	  portNumber, // start listening on this port (aka service)
	  serviceName, // well defined name for this service
	  new Class[] {SoapConnectionServer.class}
	  );
    
    // an anonymous thread, so that the SoapConnectionServer
    // doesn't die till the componentID is received. The
    // creationServiceImpl invokes doShutdown() to stop
    // this waiting thread.
    (new Thread() {
	public void run() {
	  waitForShutdown();
	}
      }).start();
    // using soaprmi api to the url at which SoapConnectionServer is waiting
    String locationWaitingAt = SoapServices.getDefault().getStartpointLocation(this);
    // using soaprmi api to the port at which SoapConnectionServer is waiting
    // this port can even be obtained from the url received above.
    portWaitingOn = SoapServices.getDefault().getStartpointLocationPort(this);
    logger.finer("listening on the port: " + port);
    logger.finer("waiting on location: " + locationWaitingAt);
  }

  public int getPortForComponentCallBack() {
    logger.finer("portForComponentCallBack is: " + portWaitingOn);
    return portWaitingOn;
  }
 
  // this is a remote method method that is invoked by
  // SoapConnectionClient
  public void sendComponentID(ComponentID componentID) throws RemoteException {
    logger.finest("called");
    if (componentID == null) {
      logger.severe("Received null componentID");
      throw new RemoteException("Received null componentID !");
    }
    componentIDofInstantiatedComponent = componentID;
    stopWaitingForComponentID(true);
  }

  /**
   * called by CreationServerImpl
   */
  public ComponentID getComponentID() {
    waitForComponentID();
    return componentIDofInstantiatedComponent;
  }

  /**
   * called from getComponentID().
   * this keeps the creationServiceImpl waiting (blocking)
   * till the componentID of the instantiated component
   * is received.
   */
  private synchronized void waitForComponentID() {
    while (!gotComponentID) {
      try {
	wait();	
      } catch ( Exception e ) {
	e.printStackTrace();
      }
    }
  }

  private synchronized void stopWaitingForComponentID(boolean stopWaiting) {
    gotComponentID = stopWaiting;
    try {
      notifyAll();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * called from the constructor. This instance of
   * SoapConnectionServer needs to be kept alive
   * till the creationService invokes doShutdown().
   */
  private synchronized void waitForShutdown() {
    while (!gotShutdown) {
      try {
	wait();	
      } catch ( Exception e ) {
	e.printStackTrace();
      }
    }
  }

  /**
   * Called by creationServiceImpl once it receives the 
   * componentID of the instantiated component.
   */
  public synchronized void doShutDown(boolean stopWaiting) {
    gotShutdown  = stopWaiting;
    try {
      notifyAll();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}
